@@ -45,7 +45,7 @@ class _LSQRprod(LinearOperator):
         self.shape = (A.shape[0], self.nbar)
         self.dtype = A.dtype
     def _matvec(self, x):
-        y = np.zeros(self.n)
+        y = np.zeros(self.n, dtype=x.dtype)
         y[self.nnz_idd] = \
             x - (1. / self.nbar) * np.dot(np.dot(np.conj(self.ebar),
                                                  x), self.ebar)
@@ -94,7 +94,7 @@ def _printf(fid, message):
 
 def _oneprojector_i(b, tau):
     n = b.size
-    x = np.zeros(n)
+    x = np.zeros(n, dtype=b.dtype)
     bNorm = np.linalg.norm(b, 1)
 
     if tau >= bNorm:
@@ -106,7 +106,7 @@ def _oneprojector_i(b, tau):
         b = b[idx]
 
         csb = np.cumsum(b) - tau
-        alpha = np.zeros(n+1)
+        alpha = np.zeros(n + 1)
         alpha[1:] = csb / (np.arange(n) + 1.0)
         alphaindex = np.where(alpha[1:] >= b)[0]
         if alphaindex.any():
@@ -120,7 +120,7 @@ def _oneprojector_i(b, tau):
 
 def _oneprojector_d(b, d, tau):
     n = b.size
-    x = np.zeros(n)
+    x = np.zeros(n, dtype=b.dtype)
 
     if tau >= np.linalg.norm(d*b, 1):
         x = b.copy()
@@ -160,11 +160,11 @@ def oneprojector(b, d, tau):
     Projects b onto the (weighted) one-norm ball of radius tau.
     If d=1 solves the problem::
 
-        minimize_x  ||b-x||_2  st  ||x||_1 <= tau.
+        minimize_x  ||b-x||_2  subject to  ||x||_1 <= tau.
 
     else::
 
-        minimize_x  ||b-x||_2  st  || Dx ||_1 <= tau.
+        minimize_x  ||b-x||_2  subject to  ||Dx||_1 <= tau.
 
     Parameters
     ----------
@@ -347,7 +347,7 @@ def _norm_l12_project(g, x, weights, tau):
     xx = spdiags(xc, 0, m, m) * xx
     return xx.flatten()
 
-def _norm_l1nn_primal(x, weights):
+def norm_l1nn_primal(x, weights):
     """Non-negative L1 gauge function
 
     Parameters
@@ -369,7 +369,7 @@ def _norm_l1nn_primal(x, weights):
         p = np.linalg.norm(x * weights, 1)
     return p
 
-def _norm_l1nn_dual(x, weights):
+def norm_l1nn_dual(x, weights):
     """Dual of non-negative L1 gauge function
 
     Parameters
@@ -390,7 +390,7 @@ def _norm_l1nn_dual(x, weights):
     p = np.linalg.norm(xx/weights, np.inf)
     return p
 
-def _norm_l1nn_project(x, weights, tau):
+def norm_l1nn_project(x, weights, tau):
     """Projection onto the non-negative part of the one-norm ball
 
     Parameters
@@ -412,7 +412,7 @@ def _norm_l1nn_project(x, weights, tau):
     xx[xx < 0] = 0
     return _norm_l1_project(xx, weights, tau)
 
-def _norm_l12nn_primal(g, x, weights):
+def norm_l12nn_primal(g, x, weights):
     """Non-negative L1 norm with weighted input vector with number
     of groups equal to g
 
@@ -442,7 +442,7 @@ def _norm_l12nn_primal(g, x, weights):
         nrm = np.sum(weights*np.sqrt(np.sum(xm**2, axis=1)))
     return nrm
 
-def _norm_l12nn_dual(g, x, weights):
+def norm_l12nn_dual(g, x, weights):
     """Dual on non-legative L1L norm with weighted input vector
     with number of groups equal to g
 
@@ -471,7 +471,7 @@ def _norm_l12nn_dual(g, x, weights):
     nrm = np.linalg.norm(np.sqrt(np.sum(xx ** 2, axis=1)) / weights, np.inf)
     return nrm
 
-def _norm_l12nn_project(g, x, weights, tau):
+def norm_l12nn_project(g, x, weights, tau):
     """Projection with number of groups equal to g
 
     Parameters
@@ -881,7 +881,7 @@ def spgl1(A, b, tau=0, sigma=0, x0=None, fid=None, verbosity=0,
     # Determine initial x and see if problem is complex
     realx = np.lib.isreal(A).all() and np.lib.isreal(b).all()
     if x0 is None:
-        x = np.zeros(n)
+        x = np.zeros(n, dtype=b.dtype)
     else:
         x = np.asarray(x0)
 
@@ -1174,7 +1174,7 @@ def spgl1(A, b, tau=0, sigma=0, x0=None, fid=None, verbosity=0,
                     # LSQR iterations successful. Take the subspace step.
                     if istop != 4:
                         # Push dx back into full space: dx = Z dx.
-                        dx = np.zeros(n)
+                        dx = np.zeros(n, dtype=x.dtype)
                         dx[nnz_idx] = \
                             dxbar - (1/nebar)*np.dot(np.dot(np.conj(ebar.T),
                                                             dxbar), dxbar)
@@ -1288,7 +1288,7 @@ def spgl1(A, b, tau=0, sigma=0, x0=None, fid=None, verbosity=0,
         elif stat == EXIT_LEAST_SQUARES:
             _printf(fid, 'EXIT -- Found a least-squares solution')
         elif stat == EXIT_LINE_ERROR:
-            _printf(fid, 'ERROR EXIT -- Linesearch error (%i)' % lnerr)
+            _printf(fid, 'ERROR EXIT -- Linesearch error (%d)' % lnerr)
         elif stat == EXIT_SUBOPTIMAL_BP:
             _printf(fid, 'EXIT -- Found a suboptimal BP solution')
         elif stat == EXIT_MATVEC_LIMIT:
